#include <stdio.h>
#include <stdlib.h>
#include <libpq-fe.h>

// gcc -o pgtest pgtest.c -I/usr/include/postgresql -lpq

int main(int argc, char **argv)
{
    const char	*conninfo;
    PGconn      *conn;
    PGresult	*res;

    /* 连接数据库 */
    conn = PQconnectdb("dbname=db_mancode user=user_mancode password=123456 hostaddr=192.168.0.64 port=5432");

    /* Check to see that the backend connection was successfully made */
    if (PQstatus(conn) != CONNECTION_OK)
    {
        fprintf(stderr, "Connection to database failed: %s", PQerrorMessage(conn));
	    
        PQfinish(conn);

        return 1;
    }

    /* 开始检索数据，开启一个事务 */
    res = PQexec(conn, "BEGIN");
    if (PQresultStatus(res) != PGRES_COMMAND_OK)
    {
        fprintf(stderr, "BEGIN command failed: %s", PQerrorMessage(conn));
        PQclear(res);
        PQfinish(conn);

        return 1;
    }
    PQclear(res);

    res = PQexec(conn, "DECLARE myportal CURSOR FOR SELECT * FROM t_map");
    if (PQresultStatus(res) != PGRES_COMMAND_OK)
    {
        fprintf(stderr, "DECLARE CURSOR failed: %s", PQerrorMessage(conn));
        PQclear(res);
        PQfinish(conn);

        return 1;
    }
    PQclear(res);
 
    res = PQexec(conn, "FETCH ALL in myportal");
    if (PQresultStatus(res) != PGRES_TUPLES_OK)
    {
        fprintf(stderr, "FETCH ALL failed: %s", PQerrorMessage(conn));
        PQclear(res);
        PQfinish(conn);

        return 1;
    }
 
    /* 先打印表头 */
    int iFields = PQnfields(res);
    for (int i = 0; i < iFields; i++)
    {
        printf("%-15s", PQfname(res, i));
    }
    printf("\n");

    /* 再打印数据 */
    for (int i = 0; i < PQntuples(res); i++)
    {
        for (int j = 0; j < iFields; j++)
        {
            printf("%-15s", PQgetvalue(res, i, j));
        }
        printf("\n");
    }

    PQclear(res);
 
    /* 关闭游标 */
    res = PQexec(conn, "CLOSE myportal");
    PQclear(res);
 
    /* 结束事务 */
    res = PQexec(conn, "END");
    PQclear(res);
 
    /* 关闭连接 */
    PQfinish(conn);

    return 0;
}
